const { runMultiFileCodeWithOptionalTestData, setGiteaRepoUrl, setGiteaToken, setTestDataPath } = require('./enhancedmultiFileRunner');
const Docker = require('dockerode');
const fs = require('fs/promises');
const path = require('path');
const axios = require('axios');

jest.mock('dockerode');
jest.mock('fs/promises');
jest.mock('axios');

const mockDocker = new Docker();
const mockContainer = {
    start: jest.fn(),
    logs: jest.fn(),
    wait: jest.fn(),
    remove: jest.fn(),
};

mockDocker.createContainer = jest.fn().mockResolvedValue(mockContainer);
mockContainer.logs.mockResolvedValue({ on: jest.fn() });
mockContainer.wait.mockResolvedValue({ StatusCode: 0 });

describe('runMultiFileCodeWithOptionalTestData', () => {
    beforeEach(() => {
        jest.clearAllMocks();
    });

    it('should throw an error if files are not provided', async () => {
        await expect(runMultiFileCodeWithOptionalTestData({ files: [], language: 'c' }))
            .rejects.toThrow('Files and language are required.');
    });

    it('should throw an error if language is not provided', async () => {
        await expect(runMultiFileCodeWithOptionalTestData({ files: [{ name: 'main.c', content: '' }], language: '' }))
            .rejects.toThrow('Files and language are required.');
    });

    it('should throw an error if an unsupported language is provided', async () => {
        await expect(runMultiFileCodeWithOptionalTestData({ files: [{ name: 'main.c', content: '' }], language: 'ruby' }))
            .rejects.toThrow("Language 'ruby' is not supported.");
    });

    it('should successfully compile and run C code', async () => {
        const result = await runMultiFileCodeWithOptionalTestData({
            files: [{ name: 'main.c', content: 'int main() { return 0; }' }],
            language: 'c',
        });

        expect(result).toHaveProperty('output');
    });

    it('should successfully compile and run Java code', async () => {
        const result = await runMultiFileCodeWithOptionalTestData({
            files: [{ name: 'Main.java', content: 'public class Main { public static void main(String[] args) {} }' }],
            language: 'java',
        });

        expect(result).toHaveProperty('output');
    });

    it('should successfully compile and run Python code', async () => {
        const result = await runMultiFileCodeWithOptionalTestData({
            files: [{ name: 'script.py', content: 'print("Hello, World!")' }],
            language: 'python',
        });

        expect(result).toHaveProperty('output');
    });

    it('should fetch test data from Gitea if configured', async () => {
        await setGiteaRepoUrl('http://example.com');
        await setGiteaToken('token');
        await setTestDataPath('testData');

        axios.get.mockResolvedValue({ data: [{ type: 'file', name: 'testData.txt', download_url: 'http://example.com/testData.txt' }] });

        const result = await runMultiFileCodeWithOptionalTestData({
            files: [{ name: 'main.c', content: '' }],
            language: 'c',
            testDataFiles: ['testData.txt'],
            expectedAnswerFiles: ['expected.txt'],
            resultOutputFile: 'result.txt',
        });

        expect(result).toHaveProperty('output');
    });

    it('should handle errors during file operations', async () => {
        fs.writeFile.mockRejectedValue(new Error('File write error'));

        await expect(runMultiFileCodeWithOptionalTestData({
            files: [{ name: 'main.c', content: '' }],
            language: 'c',
        })).rejects.toThrow('File write error');
    });

    it('should handle errors during Docker container operations', async () => {
        mockDocker.createContainer.mockRejectedValue(new Error('Docker error'));

        await expect(runMultiFileCodeWithOptionalTestData({
            files: [{ name: 'main.c', content: '' }],
            language: 'c',
        })).rejects.toThrow('Docker error');
    });

    it('should handle errors during HTTP requests', async () => {
        axios.get.mockRejectedValue(new Error('HTTP error'));

        await expect(runMultiFileCodeWithOptionalTestData({
            files: [{ name: 'main.c', content: '' }],
            language: 'c',
            testDataFiles: ['testData.txt'],
        })).rejects.toThrow('HTTP error');
    });
});
